from bifabrik.cfg.engine.ConfigContainer import ConfigContainer
from bifabrik.cfg.DataDestinationConfigurationBase import DataDestinationConfigurationBase

class TableDestinationConfiguration(DataDestinationConfigurationBase):

    def __init__(self):
        super().__init__()